@@ -182,6 +182,9 @@ within_tolerance <- function(x, y) {
   equal <- (l2x == l2y)
   equal[is.na(equal)] <- FALSE
   out <- equal
+
+  # Work around integer64 problem
+  equal[x == y] <- FALSE
   "!!!!!!DEBUG `v(abs((x[equal] - y[equal]) * 2 ^ -l2x[equal]))`"
   out[equal] <- abs((x[equal] - y[equal]) * 2 ^ -l2x[equal]) <= eps_2
   out
@@ -233,6 +236,19 @@ compute_min_sigfig <- function(x) {
   ret
 }
 
+compute_extra_sigfig <- function(x) {
+  x <- sort(abs(x))
+  delta <- diff(x)
+  x <- x[-1]
+
+  keep <- which((delta != 0) & is.finite(delta))
+  if (length(keep) == 0) {
+    return(0)
+  }
+
+  ceiling(log10(max(x[keep] / delta[keep]))) - 1
+}
+
 LOG_10 <- log(10)
 
 compute_exp <- function(x, sigfig, digits) {
